package com.community.huiju.controller;

import com.alibaba.fastjson.JSONObject;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.code.cache.AppkeyCache;
import com.community.huiju.model.TaUser;
import com.community.huiju.model.TpTicket;
import com.community.huiju.service.ITaUserService;
import com.community.huiju.vo.TaUserVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.catalina.User;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpSession;
import javax.xml.crypto.dsig.keyinfo.PGPData;
import java.util.List;

/**
 * 用户控制器
 * @author weiximei
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "用户相关操作的API",description = "用户相关操作的API")
public class UserController extends BaseController {

    @Autowired
    private ITaUserService iTaUserService;

    @ApiOperation(value = "登录", notes = "根据登录名和验证码")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "loginName(登陆名(手机号)) code(验证码)"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
    @RequestMapping(value = "/user/login/{communityId}",method = RequestMethod.POST)
    public ResponseBean login(@RequestBody String parameter,
                              @PathVariable(value = "communityId") String communityId,
                              HttpSession session){
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        ResponseBean responseBean = iTaUserService.login(jsonObject.getString("loginName"),jsonObject.getString("code"));
        TaUserVO userVO = (TaUserVO) responseBean.getData();

        if (null != userVO) {
            UserElement userElement = new UserElement();
            BeanUtils.copyProperties(userVO,userElement);
            userElement.setLoginType(Constant.APP_LOGIN_TYPE);
            session.setAttribute(Constant.APP_USER_SESSION,userElement);
            userVO.setToken(session.getId());
        }
        return responseBean;

    }
    @ApiOperation(value = "修改手机号", notes = "修改手机号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "paramets",value = "phone：登陆(手机号)，code：(手机验证码),otherUserId:(租客或者家属ID，空为业主本人)"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
            @RequestMapping(value = "/user/phone",method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody String paramets, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        Integer userId=userElement.getId();
        ResponseBean response=iTaUserService.upDateloginName(userId,paramets,userElement.getCommunityId());
        return response;
    }

    @ApiOperation(value = "查看房屋成员列表", notes = "查看房屋成员列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
    @RequestMapping(value = "/roomUserList", method = RequestMethod.GET)
    public ResponseBean accessTicket(HttpSession session){
        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        Integer userId = userElement.getId();
        ResponseBean responseBean = new ResponseBean();
        List<TaUser> taUserList = iTaUserService.getAllRoomUserList(userId);
        responseBean.addSuccess(taUserList);
        return responseBean;
    }
    @ApiOperation(value = "修改用户名和性别和身份证", notes = "修改用户名和性别和身份证")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "userName用户民和gender性别和idCard身份证"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
    @RequestMapping(value = "/user/update/info",method = RequestMethod.PUT)
    public ResponseBean updateUserNameAndGender(@RequestBody String parameter,
                                                HttpSession session){
        ResponseBean response = new ResponseBean();

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);

        TaUser user = JSONObject.parseObject(parameter,TaUser.class);
        user.setId(userElement.getId());

        response = iTaUserService.modifyUser(user);
        return response;

    }

    @ApiOperation(value = "租客或家属 启用或者停用", notes = "租客或家属 启用或者停用")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "userId(租客或者家属) type(1启用 0停用)"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
    @RequestMapping(value = "/user/dependentsOrTenants",method = RequestMethod.PUT)
    public ResponseBean updateDependentsOrTenants(@RequestBody String parameter,
                                                HttpSession session){
        ResponseBean response = new ResponseBean();

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);

        JSONObject jsonObject = JSONObject.parseObject(parameter);
        response = iTaUserService.startOrEndUsingUser(userElement.getId(),jsonObject.getInteger("userId"),jsonObject.getInteger("type"));

        return response;

    }


    @ApiOperation(value = "业主添加 租客或家属", notes = "业主添加 租客或家属")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "userName(租客或者家属 姓名) " +
                    "phone(租客或者家属 手机号) " +
                    "gender(租客或者家属 性别 1男 2女) " +
                    "type(2租客 3家属)" +
                    "idCard(身份证号)"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
    })
    @RequestMapping(value = "/user/addTenantsOrDependents",method = RequestMethod.POST)
    public ResponseBean addTenantsOrDependents(@RequestBody String parameter,
                                               HttpSession session){
        ResponseBean response = new ResponseBean();

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        response = iTaUserService.addTenantsOrDependents(jsonObject.getString("userName"),
                jsonObject.getString("phone"),jsonObject.getString("gender"),
                userElement.getId(),
                jsonObject.getInteger("type"), jsonObject.getString("idCard"));

        return response;

    }


    @ApiOperation(value = "业主修改 租客或家属", notes = "业主修改 租客或家属")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "id(租客或者家属 ID) userName(租客或者家属 姓名) " +
                    "phone(租客或者家属 手机号) " +
                    "gender(租客或者家属 性别 1男 2女) " +
                    "type(2租客 3家属)" +
                    "idCard(身份证号)"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
    })
    @RequestMapping(value = "/user/updateTenantsOrDependents",method = RequestMethod.POST)
    public ResponseBean updateTenantsOrDependents(@RequestBody String parameter,
                                               HttpSession session){
        ResponseBean response = new ResponseBean();

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        response = iTaUserService.updateTenantsOrDependents(jsonObject.getString("userName"),
                jsonObject.getString("phone"),jsonObject.getString("gender"),
                userElement.getId(),
                jsonObject.getInteger("type"),
                jsonObject.getInteger("id"),
                jsonObject.getString("idCard"));

        return response;

    }



    @ApiOperation(value = "修改用户头像", notes = "修改用户头像")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body",dataType = "String",name = "parameter",value = "headPortrait(用户头像)"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
    })
    @RequestMapping(value = "/user/modifyUserHeadPortrait",method = RequestMethod.PUT)
    public ResponseBean modifyUserHeadPortrait(@RequestBody String parameter,
                                               HttpSession session){

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        JSONObject jsonObject = JSONObject.parseObject(parameter);
        ResponseBean response = iTaUserService.modifyUserHeadPortrait(userElement.getId(),jsonObject.getString("headPortrait"));

        return response;

    }

    @ApiOperation(value = "获取用户信息", notes = "获取用户信息, 携带userId表示查询租客或者家属")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
            @ApiImplicitParam(paramType = "query",dataType = "String",name = "userId",value = "租客或者家属的ID(可选值)"),
    })
    @RequestMapping(value = "/user/info",method = RequestMethod.GET)
    public ResponseBean getUserInfo(@RequestParam(value = "userId",required = false) Integer userId,
                                    HttpSession session){
        ResponseBean response = new ResponseBean();

        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        Integer id =  userElement.getId();

        response = iTaUserService.getUserInfo(id,userId);
        TaUserVO userVO = (TaUserVO) response.getData();
        userVO.setToken(session.getId());

        return response;

    }


    @ApiOperation(value = "游客登录", notes = "游客登录")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = Integer.class, name = "communityId", value = "小区Id"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "Login-Type", value = "值为 app"),
    })
    @RequestMapping(value = "/user/temporary/{communityId}", method = RequestMethod.POST)
    public ResponseBean temporaryLogin(@PathVariable Integer communityId, HttpSession session) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iTaUserService.temporaryLogin(communityId);
        TaUserVO userVO = (TaUserVO) responseBean.getData();
        if (null != userVO) {
            UserElement userElement = new UserElement();
            BeanUtils.copyProperties(userVO,userElement);
            userElement.setLoginType(Constant.APP_LOGIN_TYPE);
            session.setAttribute(Constant.APP_USER_SESSION,userElement);
            userVO.setToken(session.getId());
        }
        userVO.setToken(session.getId());
        return responseBean;
    }

    @ApiOperation(value = "迎宾系统添加人员", notes = "迎宾系统添加人员")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "form", dataTypeClass = String.class, value = "userName人员名称"),
            @ApiImplicitParam(paramType = "header",dataType = "String",name = "X-Auth-Token",value = "Token"),
    })
    @RequestMapping(value = "/user/welcome/add", method = RequestMethod.POST)
    public ResponseBean welcomeAddUser(@RequestParam("userName") String userName, HttpSession session) {
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        TaUser user = new TaUser();
        user.setUserName(userName);
        responseBean = iTaUserService.addWelcomeUser(userElement, user);
        return responseBean;
    }

    @ApiOperation(value = "查看当前用户所有关联房产",notes = "查看当前用户所有关联房产")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = String.class, name = "X-Auth-Token", value = "Token")
    })
    @RequestMapping(value ="/userhouse" ,method = RequestMethod.GET)
    public ResponseBean userHouseList(HttpSession session){
        UserElement userElement = (UserElement) session.getAttribute(Constant.APP_USER_SESSION);
        ResponseBean responseBean= iTaUserService.userHouseList(userElement);
        return responseBean;
    }
}
