package com.community.huiju.service;

import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.TaUser;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * 用户业务 接口
 * @author weiximei
 */
public interface ITaUserService {


    /**
     * 校验 用户认证状态
     * @param user
     */
    default void checkVerifyStatusUser(TaUser user) {
        if (Constant.INVALID_VERIFY_STATUS.equals(user.getVerifyStatus())) {
            throw new WisdomException("关联账户违反社区规定，无法使用");
        }
    }

    /**
     * 用户登录
     * @param loginName
     * @param code 验证码
     * @return 用户对象
     */
    ResponseBean login(String loginName,String code);

    /**
     * 注册用户
     * @param user
     * @return 用户对象
     */
    ResponseBean register(TaUser user);

    /**
     * 修改当前用户手机号
     * @param id
     * @return
     */
    ResponseBean upDateloginName(Integer id, String paramets,Integer communityId);

    /**
     * 获取房屋成员列表
     * @param parentId
     * @return
     */
    List<TaUser> getAllRoomUserList(Integer parentId);

    /**
     * 修改用户
     * @param user
     * @return
     */
    ResponseBean modifyUser(TaUser user);

    /**
     * 停用用户 或者 启用用户
     * @param currentUserId 当前用户(业主)
     * @param userId
     * @param type 1启用 0停用
     * @return
     */
    ResponseBean startOrEndUsingUser(Integer currentUserId,Integer userId,Integer type);


    /**
     * 添加租客或者家属
     * @param userName
     * @param phone
     * @param gender
     * @param currentUserId 当前用户(业主)
     * @param type 2租客 3家属
     * @param idCard 身份证号
     * @return
     */
    ResponseBean addTenantsOrDependents(String userName, String phone, String gender,Integer currentUserId,Integer type, String idCard);

    /**
     * 修改租客或者家属
     * @param userName
     * @param phone
     * @param gender
     * @param currentUserId 当前用户(业主)
     * @param type 2租客 3家属
     * @param userId 用户ID
     * @param idCard 身份证号
     * @return
     */
    ResponseBean updateTenantsOrDependents(String userName, String phone, String gender,Integer currentUserId,Integer type,Integer userId, String idCard);

    /**
     * 修改用户头像
     * @param currentUserId
     * @param headPortrait 图片url
     * @return
     */
    ResponseBean modifyUserHeadPortrait(Integer currentUserId, String headPortrait);

    /**
     * 获取用户信息
     * @param currenUserId 当前用户
     * @param userId 被查询用户, 一般是业主情况下有效, 查的是下面的家属或者租客
     * @return
     */
    ResponseBean getUserInfo(Integer currenUserId,Integer userId);

    /**
     * 游客登录
     * @param communityId
     * @return
     */
    ResponseBean temporaryLogin(Integer communityId);

    /**
     * 迎宾专用 接口  添加人员
     * @param userElement
     * @param user
     * @param user
     * @return
     */
    ResponseBean addWelcomeUser(UserElement userElement,TaUser user);

    /**
     * 查看当前用户所有关联房产
     * @param userElement
     * @return
     */
    ResponseBean userHouseList(UserElement userElement);
}
