package com.community.huiju.service.impl;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.domain.AlipayTradeAppPayModel;
import com.alipay.api.request.AlipayTradeAppPayRequest;
import com.alipay.api.response.AlipayTradeAppPayResponse;
import com.community.commom.session.UserElement;
import com.community.huiju.common.wxpay.HfWxConfig;
import com.community.huiju.common.wxpay.WXPay;
import com.community.huiju.common.wxpay.WXPayConstants;
import com.community.huiju.common.wxpay.WXPayUtil;
import com.community.huiju.dao.TaSysRoleMapper;
import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.dao.TpBillInvoiceMapper;
import com.community.huiju.dao.TpBillMapper;
import com.community.huiju.dao.TpBillStatementMapper;
import com.community.huiju.model.TaSysRole;
import com.community.huiju.model.TaUser;
import com.community.huiju.model.TpBillInvoice;
import com.community.huiju.service.AliPayServiceI;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;

import static com.alipay.api.AlipayConstants.CHARSET_UTF8;

/**
 * @author FXF
 * @date 2018-10-25
 */
@Service("aliPayService")
@Transactional(rollbackFor = Exception.class)
public class AliPayServiceImpl implements AliPayServiceI {
	private Logger log = LoggerFactory.getLogger(AliPayServiceImpl.class);
	
	@Autowired
	private TpBillInvoiceMapper tpBillInvoiceMapper;
	
	@Autowired
	private TpBillStatementMapper tpBillStatementMapper;

	@Autowired
	private TpBillMapper tpBillMapper;

	@Autowired
	private TaUserMapper taUserMapper;

	@Autowired
	private TaSysRoleMapper taSysRoleMapper;
	
	@Value("${alipay.APP_PRIVATE_KEY}")
	private String APP_PRIVATE_KEY;
	
	@Value("${alipay.APP_ID}")
	private String ALI_APP_ID;
	
	@Value("${alipay.ALIPAY_PUBLIC_KEY}")
	private String ALIPAY_PUBLIC_KEY;
	
	/**
	 * 支付宝统一下单
	 *
	 * @return
	 * @param billInvoiceId
	 * @param userElement
	 */
	@Override
	@Transactional(rollbackFor = Exception.class)
	public String AliPayOrder(Integer billInvoiceId, UserElement userElement) throws Exception {
		Integer userId = userElement.getId();

		// 如果是家属/租客, 应该用业主的用户id
		TaUser taUser = taUserMapper.selectByPrimaryKey(userId);
		TaSysRole taSysRole = taSysRoleMapper.findRoleByUserId(userId);
		if (null != taSysRole && taSysRole.getId().intValue() != 1) {
			TaUser ownerUser = taUserMapper.selectByPrimaryKey(taUser.getParentId());
			userId = ownerUser.getId();
		}

		//获取支付金额
		TpBillInvoice tpBillInvoice = tpBillInvoiceMapper.selectByIdAndUserId(billInvoiceId,userId,userElement.getCommunityId());
		if (null == tpBillInvoice){
			throw new Exception("订单不存在");
		}else if (!tpBillInvoice.getBillStatus().equals("3")){
			throw new Exception("不是正在支付的订单");
		}
		
		//下单
		AlipayClient alipayClient = new DefaultAlipayClient("https://openapi.alipay.com/gateway.do", ALI_APP_ID, APP_PRIVATE_KEY, "json", CHARSET_UTF8, ALIPAY_PUBLIC_KEY, "RSA2");
		//实例化具体API对应的request类,类名称和接口名称对应,当前调用接口名称：alipay.trade.app.pay
		AlipayTradeAppPayRequest request = new AlipayTradeAppPayRequest();
		//SDK已经封装掉了公共参数，这里只需要传入业务参数。以下方法为sdk的model入参方式(model和biz_content同时存在的情况下取biz_content)。
		AlipayTradeAppPayModel model = new AlipayTradeAppPayModel();
		model.setBody(tpBillInvoice.getBillName());
		model.setSubject(tpBillInvoice.getBillName());
		model.setOutTradeNo(tpBillInvoice.getOutTradeNo());
		model.setTimeoutExpress("30m");
		model.setTotalAmount(BigDecimal.valueOf(Long.valueOf(tpBillInvoice.getPayPrice())).divide(new BigDecimal(100)).toString());
		model.setProductCode("QUICK_MSECURITY_PAY");
		request.setBizModel(model);
		request.setNotifyUrl("http://106.14.20.193:8086/app-api/aliPayNotify");
		try {
			//这里和普通的接口调用不同，使用的是sdkExecute
			AlipayTradeAppPayResponse response = alipayClient.sdkExecute(request);
			//就是orderString 可以直接给客户端请求，无需再做处理。
			System.out.println(response.getBody());
			return response.getBody();
		} catch (AlipayApiException e) {
			e.printStackTrace();
		}
		return null;
	}
}
