package com.community.huiju.service.impl;

import com.community.huiju.dao.TaSysRoleMapper;
import com.community.huiju.dao.TaUserMapper;
import com.community.huiju.dao.TpBillInvoiceMapper;
import com.community.huiju.model.TaSysRole;
import com.community.huiju.model.TaUser;
import com.community.huiju.service.BillServiceI;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author FXF
 * @date 2018-10-25
 */
@Service("billService")
public class BillServiceImpl implements BillServiceI {
	
	@Autowired
	private TpBillInvoiceMapper tpBillInvoiceMapper;

	@Autowired
	private TaUserMapper taUserMapper;

	@Autowired
	private TaSysRoleMapper taSysRoleMapper;
	
	/**
	 * 根据payType分页获取缴费或未缴费的数据
	 * @param communityId
	 * @param payType
	 * @param pageNum
	 * @param pageSize
	 * @return
	 */
	@Override
	public Map<String, Object> getBillsList(Integer communityId, Integer payType, Integer userId ,Integer pageNum, Integer pageSize) {
		//使用分页插件
		Page<Map<String,Object>> page = PageHelper.startPage(pageNum, pageSize);

		TaUser taUser = taUserMapper.selectByPrimaryKey(userId);

		TaSysRole taSysRole = taSysRoleMapper.findRoleByUserId(userId);
		// 如果不是业主
		if (null != taSysRole && taSysRole.getId().intValue() != 1) {
			TaUser ownerUser = taUserMapper.selectByPrimaryKey(taUser.getParentId());
			userId = ownerUser.getId();
		}

		List<Map<String,Object>> billsList = tpBillInvoiceMapper.getBillsList(communityId, userId, payType);
		billsList.forEach(e-> {
			String payPrice = String.valueOf(e.get("payPrice"));
			Double payPriceDouble = Double.parseDouble(payPrice) / 100;
			e.put("payPrice", payPriceDouble);
		});
		Map<String,Object> result = Maps.newHashMap();
		result.put("list",billsList);
		result.put("total",page.getTotal());
		return result;
	}
	
	/**
	 * 获取账单的详细信息
	 * @param billInvoiceId
	 * @return
	 */
	@Override
	public Map<String, Object> getBillInvoiceDetail(Integer communityId,Integer billInvoiceId) {
		Map<String, Object> billInvoiceDetail = tpBillInvoiceMapper.getBillInvoiceDetail(communityId, billInvoiceId);
		String payPrice = String.valueOf(billInvoiceDetail.get("payPrice"));
		Double payPriceDouble = Double.parseDouble(payPrice) / 100;
		billInvoiceDetail.put("payPrice", payPriceDouble);
		return billInvoiceDetail;
	}
}
