package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.community.commom.constant.Constant;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.commom.utils.AccountValidatorUtil;
import com.community.commom.utils.BeanTools;
import com.community.commom.utils.MD5Utils;
import com.community.commom.uuid.IdGen;
import com.community.huiju.common.code.ICode;
import com.community.huiju.common.code.cache.AppkeyCache;
import com.community.huiju.common.hk.HKConstant;
import com.community.huiju.common.hk.HKOpenApi;
import com.community.huiju.common.welcome.OkHttpRequestUtils;
import com.community.huiju.common.welcome.WelcomeProperties;
import com.community.huiju.dao.*;
import com.community.huiju.exception.WisdomException;
import com.community.huiju.model.*;
import com.community.huiju.service.ITaUserService;
import com.community.huiju.service.ImageServiceI;
import com.community.huiju.vo.TaUserVO;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.sun.org.apache.bcel.internal.generic.IF_ACMPEQ;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.logging.SimpleFormatter;

import static com.community.commom.constant.Constant.OWNER;

/**
 * 用户业务 实现
 * @author weiximei
 */
@Service("iTaUserService")
@Slf4j
public class TaUserServiceImpl implements ITaUserService {

    private IdGen idGen = IdGen.get();

    @Autowired
    private WelcomeProperties welcomeProperties;

    @Autowired
    private TaUserMapper taUserMapper;

    @Autowired
    private TpBuildingOwnerInfoMapper tpBuildingOwnerInfoMapper;

    @Autowired
    private TaSysRoleMapper taSysRoleMapper;

    @Autowired
    private TaSysUserRoleMapper taSysUserRoleMapper;

    @Autowired
    private ToCommunitiesMapper toCommunitiesMapper;

    @Autowired
    private SysNationMapper sysNationMapper;

    @Autowired
    private TaFaceMapper taFaceMapper;

    @Autowired
    private TpBillInvoiceMapper tpBillInvoiceMapper;

    @Autowired
    private ImageServiceI imageService;

    @Autowired
    @Qualifier("phoneCodeImpl")
    private ICode iCode;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean login(String loginName,String code) {
        ResponseBean response = new ResponseBean();

        // 校验手机号和验证码
        iCode.checkPhoneAndCode(loginName, code);

        Map<String,Object> map = Maps.newHashMap();
        map.put("loginName",loginName);
        TaUser currentUser = taUserMapper.selectByLoginName(map);
        if (null == currentUser) {
            response.addError("未注册，请先注册");
            return response;
        }

        // 校验用户
        checkVerifyStatusUser(currentUser);

        // 查询房产


        TaUserVO taUserVO = new TaUserVO();
        BeanUtils.copyProperties(currentUser, taUserVO);
        response.addSuccess(taUserVO);
        return response;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseBean register(TaUser user) {

        // TODO 注册逻辑需要完善
        /**
         * 1.自动生成账号的基本信息
         * 2.匹配资料库 已认证 未认证
         * 2.角色类型
         *
         * 具体看原型图
         *
         */

        ResponseBean<TaUserVO> response = new ResponseBean();
        // 开始注册
        user.setHkPersonNo(idGen.nextId());
        user.setCreateDate(new Date());
        user.setUpdateDate(new Date());
        user.setStatus("1");
        user.setAcceptAgreementStatus("1");
        user.setUserName(user.getLoginName());
        user.setRemark("这是系统自动注册!");
        user.setVerifyStatus("0");
        user.setFaceStatus("0");
        //user.setLoginPassword(MD5Utils.encode(user.getLoginPassword()));
        // 0表示 未选择性别
        user.setGender("0");
        taUserMapper.insertSelective(user);

        user.setCreateUser(user.getId());
        user.setUpdateUser(user.getId());

        // 这里先默认 type=4 (未认证)
        int roleType = 4;

        // 查询资料库
        // TODO 这里默认List接收, 后期会有多个小区认证,可能会返回多条
        List<TpBuildingOwnerInfo> buildingOwnerInfoList = tpBuildingOwnerInfoMapper.getByCommunityIdAndOwnerTel(user.getCommunityId(), user.getLoginName());
        if (CollectionUtils.isNotEmpty(buildingOwnerInfoList)) {
            // TODO 目前是一个业主对应单个小区, 所以取第一个
            TpBuildingOwnerInfo buildingOwnerInfo = buildingOwnerInfoList.get(0);
            user.setBuildingOwnerInfoId(buildingOwnerInfo.getId());
            user.setVerifyStatus("1");
            user.setUserName(buildingOwnerInfo.getOwnerName());
            // 标记位1, 业主身份
            roleType = 1;

            // 更新资料库的关联状态 和 认证状态
            buildingOwnerInfo.setVerifyStatus("1");
            buildingOwnerInfo.setPairStatus("1");
            buildingOwnerInfo.setUpdateDate(new Date());
            tpBuildingOwnerInfoMapper.updateByPrimaryKeySelective(buildingOwnerInfo);

            // 开始绑定缴费单
            List<TpBillInvoice> tpBillInvoices = tpBillInvoiceMapper.selectByBuildingOwnerInfoId(buildingOwnerInfo.getId());
            if (CollectionUtils.isNotEmpty(tpBillInvoices)) {
                Map<String, Object> map = new HashMap<>();
                map.put("taUserId", user.getId());
                map.put("list", tpBillInvoices);
                tpBillInvoiceMapper.updateBeach(map);
            }

        }

        // 更新
        taUserMapper.updateByPrimaryKeySelective(user);

        // 这里默认 roleType=4 (未认证),  资料库认证后, 变为 1 (业主)
        // 资料库已经对接, 新注册的默认为 未认证, 只有匹配了资料库后才是业主
        addOrUpdateUserRole(roleType,response,user,true);


        // 推送海康
        /**
         * 1.获取部门(比如 住户)
         * 2.根据部门编号推送 海康
         */
        addUserAndOpenCard(response,user);

        // 给用户添加 海康门禁权限
        HKOpenApi.addAuthoritiesByPersonIds(String.valueOf(user.getHkUserId()));

        // 下发门禁权限
        HKOpenApi.downloadAuthorityByDeviceUuids();

        //--------- 可视对讲 ----------
        visualIntercom(user);


        TaUserVO taUserVO = new TaUserVO();
        BeanUtils.copyProperties(user,taUserVO);
        response.addSuccess(taUserVO);
        return response;
    }

    /**
     * 可视对讲下发
     * @param user
     */
    private void visualIntercom(TaUser user){
        // TODO 目前下发的 authName权限组是我们自定义的, 后面肯定是按照小区的 期楼栋号来设置的
        // TODO longNums设备编号, 可视对讲的机器编号, 应该根据 期楼栋号 下面的机器编号来设置的

        //--------- 可视对讲 ----------

        // 可视对讲 根据人员 ID 集添加权限并下发
        Map<String, Object> addOutDoorAuthMap = new HashMap<>();
        addOutDoorAuthMap.put("authName", "荟居");
        addOutDoorAuthMap.put("personIds", Arrays.asList(user.getHkUserId()));
        addOutDoorAuthMap.put("longNums", Arrays.asList("10010100000"));
        // 可视对讲 根据人员 ID 集添加权限并下发
        HKOpenApi.addOutDoorAuth(addOutDoorAuthMap);

        // 可视对讲 指定人员指纹权限下载
//        Map<String, Object> downloadFingerMap = new HashMap<>();
//        // opType 操作类型 Integer 是 1 为下发权限；2 为删除权限
//        downloadFingerMap.put("opType", 1);
//        downloadFingerMap.put("personIds", Arrays.asList(user.getHkUserId()));
//        downloadFingerMap.put("longNums", Arrays.asList("10010100000"));
//        // 可视对讲 指定人员指纹权限下载
//        HKOpenApi.downloadFinger(downloadFingerMap);

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseBean upDateloginName(Integer id, String paramets,Integer communityId) {
        ResponseBean response = new ResponseBean();

        JSONObject jsonObject = JSONObject.parseObject(paramets);
        String phone = (String) jsonObject.get("phone");
        String code = (String) jsonObject.get("code");
        String otherUserId = (String) jsonObject.get("otherUserId");

        // 默认为当前用户
        Integer userId = id;

        if (StringUtils.isNotBlank(otherUserId)) {
            /**校验otherUserId是否为当前登入用户的家属和业主**/
            response=getTaFaceParentId( id, Integer.valueOf(otherUserId));
            if ("1".equals(response.getCode())) {
                return response;
            }
            userId = Integer.valueOf(otherUserId.trim());
        }

        TaUser user=taUserMapper.selectByPrimaryKey(userId);
        //系统验证码
        String  codes= (String) AppkeyCache.getCache(phone);
        //用户电话
            /*String  phones=user.getLoginName();*/
        if (!AccountValidatorUtil.isPhone(phone)){
            response.addError("请输入正确的手机号!");
            return response;
        }
        //当前小区下的电话校验
        TaUser loginName=taUserMapper.getByLoginName(user.getCommunityId(),phone);
        if (null!=loginName)  {
            response.addError("号码已占用");
            return response;
        }
        String data=HKOpenApi.HKpersonPhone(user.getHkUserId(),user.getHkPersonNo(),user.getUserName(),phone);
        JSONObject jsonobject = JSONObject.parseObject(data);
        int errorCode = jsonobject.getInteger("errorCode");
        if (errorCode!=0){
            response.addError("海康电话已存在,推送海康电话修改失败");
            return response;
        }

        if ("1234".equals(code)) {

        }else if (StringUtils.isBlank(code) || !codes.equals(code)) {
            response.addError("验证码错误!");
            return response;
        }
            TaUser taUser=new TaUser();
            taUser.setLoginName(phone);
            taUser.setId(userId);
            taUser.setUpdateDate(new Date());
            taUser.setUpdateUser(id);
            taUserMapper.updateLongName(taUser);
            response.addSuccess("修改成功");
            AppkeyCache.setCache(user.getLoginName(),"null");


            /**
             * 如果是家属或者租客,修改手机号, 就不允许修改资料库的信息
             */

            // 修改楼栋信息资料库的手机号
            Map<String, Object> roleMap = Maps.newHashMap();
            roleMap.put("communityId", user.getCommunityId());
            roleMap.put("userId", userId);
            TaSysUserRole taSysUserRole = taSysUserRoleMapper.selectByCommunityIdAndUserId(roleMap);
            // 1 业主 2 租客 3 家属
            // 只允许业主修改
            if (taSysUserRole.getRoleId().intValue() == 1) {
                // TODO 后期可能会有一个业主对应多个小区, 也就是会有多个资料, 具体看需求
                // TODO 目前是一对一
                TpBuildingOwnerInfo buildingOwnerInfo = tpBuildingOwnerInfoMapper.selectByPrimaryKey(user.getBuildingOwnerInfoId());
                buildingOwnerInfo.setOwnerTel(phone);
                tpBuildingOwnerInfoMapper.updateByPrimaryKeySelective(buildingOwnerInfo);
            }
            response.addSuccess(new String[0]);
            return response;
    }

    public ResponseBean getTaFaceParentId(Integer userId,Integer otherUserId){
        ResponseBean  responseBean= new ResponseBean();
        TaUser  users= taUserMapper.selectByPrimaryKey(otherUserId);
        if (null == users || !userId.equals(users.getParentId())){
            responseBean.addError("您输入的家属或租户ID有误");
            return responseBean;
        }
        return responseBean;
    }

    @Override
    public List<TaUser> getAllRoomUserList(Integer parentId) {
        List<TaUser> taUserList = taUserMapper.selectUserListByParentId(parentId);
        taUserList.stream().forEach(e->{
            // 角色
            TaSysRole taSysRole = taSysRoleMapper.findRoleByUserId(e.getId());
            e.setRoleName(taSysRole.getRoleName());
            e.setRole(taSysRole.getId());
        });

        return taUserList;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseBean modifyUser(TaUser user) {

        ResponseBean response = new ResponseBean();

        TaUser taUser = taUserMapper.selectByPrimaryKey(user.getId());
        HashMap map= new HashMap<>();
                map.put("communityId",taUser.getCommunityId());
                map.put("userId",taUser.getId());
        TaSysUserRole TaSysUserRole= taSysUserRoleMapper.selectByCommunityIdAndUserId(map);
        BeanTools.copyProperties(user, taUser);
            int result = taUserMapper.updateByPrimaryKeySelective(taUser);
            int VerifyStatus= Integer.parseInt(taUser.getVerifyStatus());
            int Role= TaSysUserRole.getRoleId();
            if (1==VerifyStatus &&1==Role) {
            TpBuildingOwnerInfo TpBuildingOwnerInfo = new TpBuildingOwnerInfo();
            TpBuildingOwnerInfo.setOwnerName(user.getUserName());
            TpBuildingOwnerInfo.setGender(user.getGender());
            TpBuildingOwnerInfo.setOwnerTel(user.getLoginName());
            TpBuildingOwnerInfo.setUpdateDate(new Date());
            TpBuildingOwnerInfo.setUpdateUser(taUser.getId());
            TpBuildingOwnerInfo.setId(taUser.getBuildingOwnerInfoId());
            tpBuildingOwnerInfoMapper.updateByPrimaryKeySelective(TpBuildingOwnerInfo);
        }
        if (result > 0){
            response.addSuccess("修改成功!");
        }else {
            response.addError("修改失败!");
        }

        return response;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseBean startOrEndUsingUser(Integer currentUserId, Integer userId, Integer type) {
        ResponseBean response = null;
        if (Constant.EFFECTIVE.equals(type+"")){
            response = startAndEnd(currentUserId,userId,true);
        } else {
            response = startAndEnd(currentUserId,userId,false);
        }
        return response;
    }

    /**
     *  用户停用和启用
     * @param currentUserId 当前用户(业主)
     * @param userId 租客或者家属
     * @param bool 启用为 true  停用为 false
     * @return
     */
    private ResponseBean startAndEnd(Integer currentUserId, Integer userId, boolean bool){

        // TODO 更新人字段需要待定
        /**
         * 更新人 是业主 还是 物业
         */

        ResponseBean response = new ResponseBean();

        TaUser user = taUserMapper.selectByPrimaryKey(userId);
        if (null == user) {
            response.addError("该用户不存在!");
            return response;
        }
        if (!currentUserId.equals(user.getParentId())) {
            response.addError("该用户不是您的家属或租客!");
            return response;
        }
        if (bool) {
            if (Constant.EFFECTIVE.equals(user.getStatus())) {
                response.addError("用户已启用!");
                return response;
            }
            user.setStatus(Constant.EFFECTIVE+"");
        } else {
            if (Constant.INVALID.equals(user.getStatus())) {
                response.addError("用户已停用!");
                return response;
            }
            user.setStatus(Constant.INVALID+"");
        }

        int result = taUserMapper.updateByPrimaryKeySelective(user);
        if (result > 0) {
            response.addSuccess(bool==true?"用户启用成功!":"用户停用成功!");
        } else {
            response.addError(bool==true?"用户启用失败!":"用户停用失败!");
        }

        return response;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ResponseBean addTenantsOrDependents(String userName, String phone, String gender, Integer currentUserId, Integer type, String idCard) {
        ResponseBean response = new ResponseBean();

        TaUser taUser = taUserMapper.selectByPrimaryKey(currentUserId);
        if (null == taUser) {
            response.addError("当前用户不存在!");
            return response;
        }
        if (taUser.getVerifyStatus().equals("0") || taUser.getVerifyStatus().equals("2")) {
            response.addError("对不起,您未认证或认证已作废!");
            return response;
        }
        // 当前用户的身份
        TaSysRole role = taSysRoleMapper.findRoleByUserId(taUser.getId());
        if (!"OWNER".equals(role.getRoleName())) {
            response.addError("对不起,您不是业主,不能添加家属或租客!");
            return response;
        }

        int tempType = type;
        if (tempType != 2 && type != 3) {
            response.addError("对不起! 您只能添加家属或者租客!");
            return response;
        }

        if (StringUtils.isEmpty(idCard)) {
            response.addError("对不起! 身份证不允许为空!");
            return response;
        }

        Map<String,Object> parUserLogin = Maps.newHashMap();
        parUserLogin.put("loginName",phone);
        TaUser parUser = taUserMapper.selectByLoginName(parUserLogin);
        if (null != parUser) {
            response.addError("手机号 "+phone + " 已存在!");
            return response;
        }

        TaUser user = new TaUser();
        user.setHkPersonNo(idGen.nextId());
        user.setUserName(userName);
        user.setLoginName(phone);
        user.setGender(gender);
        user.setCreateDate(new Date());
        user.setCreateUser(currentUserId);
        user.setUpdateDate(new Date());
        user.setUpdateUser(currentUserId);
        user.setParentId(currentUserId);
        user.setStatus(Constant.EFFECTIVE+"");
        user.setVerifyStatus("1");
        user.setAcceptAgreementStatus("1");
        user.setFaceStatus("0");
        user.setRemark("这是业主添加的!");
        user.setIdCard(idCard);
        if (StringUtils.isEmpty(userName)) {
            user.setUserName(user.getLoginName());
        }
        user.setBuildingOwnerInfoId(taUser.getBuildingOwnerInfoId());
        user.setCommunityId(taUser.getCommunityId());

        int result = taUserMapper.insertSelective(user);
        if (result < 0) {
            response.addError("添加失败!");
            return response;
        }
        // 用户角色添加到本地库
        addOrUpdateUserRole(type,response,user,true);

        // 推送海康
        /**
         * 1.获取部门(比如 住户)
         * 2.根据部门编号推送 海康
         */
        addUserAndOpenCard(response,user);

        // 给用户添加 海康门禁权限
        HKOpenApi.addAuthoritiesByPersonIds(String.valueOf(user.getHkUserId()));

        // 下发门禁权限
        HKOpenApi.downloadAuthorityByDeviceUuids();

        //--------- 可视对讲 ----------
        visualIntercom(user);

        if ("0".equals(response.getCode())) {
            response.addSuccess(user);
        }

        return response;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ResponseBean updateTenantsOrDependents(String userName, String phone, String gender, Integer currentUserId, Integer type,Integer userId, String idCard) {
        ResponseBean response = new ResponseBean();

        TaUser taUser = taUserMapper.selectByPrimaryKey(currentUserId);
        if (null == taUser) {
            response.addError("当前用户不存在!");
            return response;
        }
        if (taUser.getVerifyStatus().equals("0") || taUser.getVerifyStatus().equals("2")) {
            response.addError("对不起,您未认证或认证已作废!");
            return response;
        }
        // 当前用户的身份
        TaSysRole role = taSysRoleMapper.findRoleByUserId(taUser.getId());
        if (!"OWNER".equals(role.getRoleName())) {
            response.addError("对不起,您不是业主,不能修改家属或租客!");
            return response;
        }

        int tempType = type;
        if (tempType != 2 && type != 3) {
            response.addError("对不起! 您只能修改家属或者租客!");
            return response;
        }

        if (StringUtils.isEmpty(idCard)) {
            response.addError("对不起! 身份证不允许为空!");
            return response;
        }

        TaUser user = taUserMapper.selectByPrimaryKey(userId);
        user.setUserName(userName);
        user.setLoginName(phone);
        user.setGender(gender);
        user.setUpdateDate(new Date());
        user.setUpdateUser(currentUserId);
        user.setParentId(currentUserId);
        user.setIdCard(idCard);
        if (StringUtils.isEmpty(userName)) {
            user.setUserName(user.getLoginName());
        }
        user.setBuildingOwnerInfoId(taUser.getBuildingOwnerInfoId());
        user.setCommunityId(taUser.getCommunityId());

        int result = taUserMapper.updateByPrimaryKeySelective(user);
        if (result < 0) {
            response.addError("修改失败!");
            return response;
        }
        // 修改用户
        addOrUpdateUserRole(type,response,user,false);

        // 推送海康
        updateUser(response,user);

        // 下发门禁权限
        HKOpenApi.downloadAuthorityByDeviceUuids();

        return response;
    }

    /**
     * 公共方法
     *  添加人员, 开卡
     * @param response
     * @param user
     */
    private void addUserAndOpenCard(ResponseBean response,TaUser user){
        Map<String,Object> parDept = Maps.newHashMap();
        parDept.put("pageNo",1);
        parDept.put("pageSize",100);
        parDept.put("deptName",Constant.DEPT_RESIDENTS);
        // 部门UUID
        String deptUuid = getDeptUUID(parDept);

        // 添加人员
        Map<String,Object> parUser = Maps.newHashMap();
        parUser.put("personNo", user.getHkPersonNo());
        parUser.put("personName",user.getUserName());
        parUser.put("phoneNo",user.getLoginName());
        parUser.put("deptUuid",deptUuid);
        Map<String,Object> resultMap = JSONObject.parseObject(HKOpenApi.addUser(parUser),HashMap.class);
        int errorCode = (int) resultMap.get("errorCode");
        if (errorCode == 0) {
            Map<String,Object> resultDataMap = (Map<String, Object>) resultMap.get("data");
            // 开卡 卡号
            long cardNo = System.currentTimeMillis();
            // 海康人员ID
            Integer personId = (Integer) resultDataMap.get("personId");
            TaUser tempUser = taUserMapper.selectByPrimaryKey(user.getId());
            tempUser.setHkUserId(personId);
            tempUser.setHkCardNo(cardNo+"");

            user.setHkUserId(personId);
            // 存储海康人员ID
            int row = taUserMapper.updateByPrimaryKeySelective(tempUser);
            if (row > 0) {
                response.addSuccess("操作成功!");
            }else {
                response.addError("操作失败");
                throw new RuntimeException("数据库添加 家属/租客 人员失败!");
            }

            // 开卡
            List<String> cardAndPersonList = Lists.newArrayList();
            cardAndPersonList.add(cardNo + "," + personId);
            response = openCard(cardAndPersonList,null);
        }else {
            String errorMessage = String.valueOf(resultMap.get("errorMessage"));
            response.addError(errorMessage);
            log.error("海康添加人员失败! {}",errorMessage);
            throw new RuntimeException(errorMessage);
        }
    }

    /**
     * 公共方法
     *  修改人员
     * @param response
     * @param user
     */
    private void updateUser(ResponseBean response,TaUser user){
        Map<String,Object> parDept = Maps.newHashMap();
        parDept.put("pageNo",1);
        parDept.put("pageSize",100);
        parDept.put("deptName",Constant.DEPT_RESIDENTS);
        // 部门UUID
        String deptUuid = getDeptUUID(parDept);

        // 人员参数
        Map<String,Object> parUser = Maps.newHashMap();
        parUser.put("personNo",user.getHkPersonNo());
        parUser.put("personId",user.getHkUserId());
        parUser.put("personName",user.getUserName());
        parUser.put("phoneNo",user.getLoginName());
        parUser.put("deptUuid",deptUuid);
        Map<String,Object> resultMap = JSONObject.parseObject(HKOpenApi.updateUser(parUser),HashMap.class);
        int errorCode = (int) resultMap.get("errorCode");
        if (errorCode == 0) {
            response.addSuccess("操作成功!");
        }else {
            String errorMessage = String.valueOf(resultMap.get("errorMessage"));
            response.addError(errorMessage);
            log.error("海康修改人员失败! {}",errorMessage);
            throw new RuntimeException(errorMessage);
        }
    }

    /**
     * 获取部门 uuid
     *
     *  这里的部门相当于 小区
     *
     * @return
     */
    private String getDeptUUID(Map<String,Object> parDept){
        Map<String,Object> deptMap = JSONObject.parseObject(HKOpenApi.getDeptName(parDept), HashMap.class);
        Map<String,Object> deptDataMap = (Map<String, Object>) deptMap.get("data");
        JSONArray jsonArray = (JSONArray) deptDataMap.get("list");
        Map<String,Object> deptListData = (Map<String, Object>) jsonArray.get(0);
        // 部门UUID
        String deptUuid = (String) deptListData.get("deptUuid");
        return deptUuid;
    }

    /**
     * 批量开卡
     * @param cardIdAndHKUserID list.add("卡号,人员ID")
     * @param expirationTime 有效期为多少? 单位为 年
     * @return
     */
    private ResponseBean openCard(List<String> cardIdAndHKUserID, Integer expirationTime){
        ResponseBean response = new ResponseBean();

        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd");
        LocalDate today = LocalDate.now();
        int year = today.getYear();
        // 加上20, 表示加上20年,默认
        if (null == expirationTime || expirationTime <= 0) {
            year += 20;
        } else {
            year += expirationTime;
        }
        int month = today.getMonthValue();
        int day = today.getDayOfMonth();

        Long time = null;
        try {
            time = formatter.parse(year + "-" + month + "-" + day).getTime();
        } catch (ParseException e) {
            e.printStackTrace();
        }

        // 开卡
        Map<String,Object> parOpenCard = Maps.newHashMap();
        parOpenCard.put("cardAndPerson",cardIdAndHKUserID);
        parOpenCard.put("startTime",System.currentTimeMillis());
        parOpenCard.put("endTime",time);
        JSONObject openCardJSON = JSONObject.parseObject(HKOpenApi.openCard(parOpenCard));
        int code = (int) openCardJSON.get("errorCode");
        if (code == 0) {
            response.addSuccess("操作成功!");
        }else {
            String errorMessage = String.valueOf(openCardJSON.get("errorMessage"));
            response.addError(errorMessage);
            log.error("开卡失败! {}",errorMessage);
            throw new RuntimeException(errorMessage);
        }

        return response;
    }

    /**
     * 公共方法,添加或者修改 角色.
     *      用于注册 和 业主添加家属和租客
     *
     *      用户登录注册的时候,没有匹配到 资料库, 就定为 未认证角色!
     *
     * @param type 1是业主 2 是租客  3 是家属 4 未认证
     * @param response
     * @param user
     * @param bool true 添加 false 修改
     */
    private void addOrUpdateUserRole(Integer type,ResponseBean response,TaUser user, boolean bool){
        // 先查询角色 在角色关联记录
        // 这里的 2 或者 3 或者 4 是对应角色编号
        int typeId = type.intValue();
        int roleType = typeId == 1 ? 1 : typeId == 2 ? 2 : typeId == 3 ? 3 : 4;
        TaSysRole taSysRole = taSysRoleMapper.selectByPrimaryKey(roleType);
        if (null == taSysRole) {
            response.addError("该权限不存在!");
            log.error("这个角色没有找到! 角色编号:{}" ,roleType);
            return;
        }

        TaSysUserRole taSysUserRole = new TaSysUserRole();
        taSysUserRole.setCommunityId(user.getCommunityId());
        taSysUserRole.setRoleId(taSysRole.getId());
        taSysUserRole.setUserId(user.getId());

        String resultSuccess = bool == true?"添加成功!":"修改成功!";
        String errorSuccess = bool == true?"业主或者租客或者家属或者未认证角色权限添加失败":"业主或者租客或者家属或者未认证角色权限修改失败";
        try {
            int resultRole = 0;
            if (bool) {
                resultRole = taSysUserRoleMapper.insertSelective(taSysUserRole);
            }else {
                Map<String,Object> map = Maps.newHashMap();
                map.put("communityId",user.getCommunityId());
                map.put("userId",user.getId());
                // 查询角色出数据
                TaSysUserRole userRole = taSysUserRoleMapper.selectByCommunityIdAndUserId(map);
                userRole.setRoleId(taSysRole.getId());
                // 修改角色
                resultRole = taSysUserRoleMapper.updateByPrimaryKeySelective(userRole);
            }
            if (resultRole > 0) {
                response.addSuccess(resultSuccess);
            } else {
                throw new RuntimeException(errorSuccess);
            }
        } catch (Exception e) {
            response.addSuccess(resultSuccess);
            log.error(errorSuccess,e);
            throw new RuntimeException(errorSuccess);
        }
    }

    @Transactional
    @Override
    public ResponseBean modifyUserHeadPortrait(Integer currentUserId, String headPortrait) {

        ResponseBean response = new ResponseBean();

        TaUser user = taUserMapper.selectByPrimaryKey(currentUserId);
        user.setHeadPortrait(headPortrait);

        int result = taUserMapper.updateByPrimaryKeySelective(user);
        if (result > 0) {
            response.addSuccess("修改成功!");
        } else {
          response.addError("修改失败!");
        }

        return response;
    }

    @Override
    public ResponseBean getUserInfo(Integer currentUserId,Integer userId) {

        ResponseBean response = new ResponseBean();
        if (null == currentUserId) {
            response.addError("用户id不存在!");
            return response;
        }

        // 获取用户
        TaUser user = taUserMapper.selectByPrimaryKey(currentUserId);
        TaUser todUser = null;
        if (null != userId) {
            todUser = taUserMapper.selectByPrimaryKey(userId);
            // 判断 家属或者租客 属不属于这个业主的下面
            if (todUser != null && user.getId().equals(todUser.getParentId())) {
                user = todUser;
            } else {
                response.addError("您的下面没有该家属或者租客!");
                return response;
            }
        }


        TaUserVO userVO = new TaUserVO();

        // 根据用户id, 获取 用户的住址, 人脸是否认证, 角色
        userSetUp(user, userVO);

        response.addSuccess(userVO);

        return response;
    }

    /**
     * 根据用户id, 获取 用户的住址, 人脸是否认证, 角色
     * @param user
     * @param userVO
     */
    private TaUserVO userSetUp(TaUser user, TaUserVO userVO){

        BeanTools.copyProperties(user,userVO);
        // 获取业主信息
        TpBuildingOwnerInfo tpBuildingOwnerInfo = null;
        if ( null != user.getBuildingOwnerInfoId() && !"".equals(user.getBuildingOwnerInfoId())) {
            // 获取业主信息
            tpBuildingOwnerInfo = tpBuildingOwnerInfoMapper.selectByPrimaryKey(user.getBuildingOwnerInfoId());
            userVO.setAddressBuilding(tpBuildingOwnerInfo.getPhase() + tpBuildingOwnerInfo.getBuilding()
                    +tpBuildingOwnerInfo.getUnit()
                    +tpBuildingOwnerInfo.getLevel()
                    +tpBuildingOwnerInfo.getRoomNo());
        }


        // 获取小区
        ToCommunities toCommunities = toCommunitiesMapper.selectByPrimaryKey(user.getCommunityId());

        // 省
        SysNation province = sysNationMapper.selectByPrimaryKey(toCommunities.getProvinceId());
        // 市
        SysNation city = sysNationMapper.selectByPrimaryKey(toCommunities.getCityId());
        // 区
        SysNation district = sysNationMapper.selectByPrimaryKey(toCommunities.getDistrictId());

        userVO.setAddressProvince(province.getProvince()+""+city.getCity()+""+district.getDistrict() + toCommunities.getCommunityName());
        userVO.setCommunityName(toCommunities.getCommunityName());

        // 角色
        TaSysRole taSysRole = taSysRoleMapper.findRoleByUserId(user.getId());
        userVO.setRole(taSysRole.getRoleName());
        // 人脸是否已录入
        // 这里目前定死为已录入, 后期根据人脸业务做判断
        if ("1".equals(user.getFaceStatus())) {
            userVO.setFace("register");
        } else {
            userVO.setFace("unregister");
        }

        if (null!=user.getParentId() && !"".equals(user.getParentId())) {
            //查询当前人业主或租客的信息
            TaUser proprietorUser = taUserMapper.selectByPrimaryKey(user.getParentId());
            userVO.setProprietorID(proprietorUser.getId());
            userVO.setProprietorPhone(proprietorUser.getLoginName());
            userVO.setProprietorName(proprietorUser.getUserName());
        }

        return userVO;
    }

    @Override
    public ResponseBean temporaryLogin(Integer communityId) {
        ResponseBean responseBean = new ResponseBean();
        ToCommunities communities = toCommunitiesMapper.selectByPrimaryKey(communityId);
        if (null == communities) {
            responseBean.addError("小区不存在!");
            return responseBean;
        }

        // 游客用户
        TaUser user = taUserMapper.selectByPrimaryKey(-1);

        TaUserVO taUserVO = new TaUserVO();
        BeanUtils.copyProperties(user, taUserVO);
        taUserVO.setCommunityId(communityId);
        taUserVO.setRole("游客");
        taUserVO.setCommunityName(communities.getCommunityName());

        responseBean.addSuccess(taUserVO);

        return responseBean;
    }

    @Override
    public ResponseBean addWelcomeUser(UserElement userElement, TaUser user) {
        ResponseBean responseBean = new ResponseBean();

        TaUser currentUser = taUserMapper.selectByPrimaryKey(userElement.getId());

        user.setBuildingOwnerInfoId(currentUser.getBuildingOwnerInfoId());
        user.setCommunityId(userElement.getCommunityId());
        user.setParentId(userElement.getId());
        user.setCreateDate(new Date());
        user.setCreateUser(userElement.getId());
        user.setCommunityId(userElement.getCommunityId());
        user.setVerifyStatus("1");
        user.setAcceptAgreementStatus("1");
        user.setFaceStatus("0");
        user.setStatus("1");

        int rows = taUserMapper.insertSelective(user);
        if (rows <= 0) {
            responseBean.addError("操作失败！");
            return responseBean;
        }

        // 角色  2 表示租客
        addOrUpdateUserRole(2,responseBean,user,true);
        responseBean.addSuccess(user);
        return responseBean;
    }

    @Override
    public ResponseBean userHouseList(UserElement userElement) {
        ResponseBean response= new ResponseBean();
        List<TaUser> taUser= taUserMapper.getByLoginNameList(userElement.getLoginName());
        return null;
    }
}
