package com.community.huiju.controller;


import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.common.base.BaseController;
import com.community.huiju.model.TpBuildingOwnerInfo;
import com.community.huiju.service.IBuildingOwnerInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpSession;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 楼栋业主资料信息表 前端控制器
 * </p>
 *
 * @author weiximei
 * @since 2018-12-18
 */
@RestController
@RefreshScope
@RequestMapping("/")
@Api(value = "楼栋业主资料信息", description = "楼栋业主资料信息 Api")
public class BuildingOwnerInfoController extends BaseController {

    @Autowired
    private IBuildingOwnerInfoService iBuildingOwnerInfoService;

    @ApiOperation(value = "获取楼栋业主信息", notes = "根据条件获取")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter",value = "building栋;" +
                    "unit单元;level楼层;roomNo户号;" +
                    "ownerName业主姓名;" +
                    "pageNum第几页;pageSize 一页多少行;")
    })
    @RequestMapping(value = "/building/list", method = RequestMethod.POST)
    public ResponseBean getList(@RequestBody String parameter, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.listQuery(parameter, userElement);
        return responseBean;
    }

    @ApiOperation(value = "获取楼栋业主信息详情", notes = "获取楼栋业主信息详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "id",value = "楼栋业主信息Id")
    })
    @RequestMapping(value = "/building/{id}", method = RequestMethod.GET)
    public ResponseBean getById(@PathVariable(value = "id") Integer id) {
        ResponseBean responseBean = new ResponseBean();
        responseBean.addSuccess(iBuildingOwnerInfoService.getById(id));
        return responseBean;

    }

    @ApiOperation(value = "修改楼栋业务信息", notes = "修改楼栋业主信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter",value = "id编号;building栋;" +
                    "unit单元;level楼层;roomNo户号;" +
                    "ownerName业主姓名;" +
                    "ownerTel业主手机号"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "X-Auth-Token", value = "token")
    })
    @RequestMapping(value = "/building/update", method = RequestMethod.PUT)
    public ResponseBean update(@RequestBody String parameter, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.update(parameter, userElement);
        return responseBean;
    }

    @ApiOperation(value = "添加楼栋业务信息", notes = "添加楼栋业务信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "building栋;" +
                    "unit单元;level楼层;roomNo户号;" +
                    "ownerName业主姓名;" +
                    "ownerTel业主手机号"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "X-Auth-Token", value = "token")
    })
    @RequestMapping(value = "/building/add", method = RequestMethod.POST)
    public ResponseBean add(@RequestBody String parameter, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.add(parameter, userElement);
        return responseBean;
    }

    @ApiOperation(value = "批量删除楼栋业主信息", notes = "批量删除楼栋业主信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "ids",value = "id集合"),
            @ApiImplicitParam(paramType = "header", dataTypeClass = String.class, name = "X-Auth-Token", value = "token")
    })
    @RequestMapping(value = "/building/delete", method = RequestMethod.DELETE)
    public ResponseBean delete(@RequestBody List<Integer> ids, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.deleteIds(ids, userElement.getId());
        return responseBean;
    }

    @ApiOperation(value = "查询 楼栋/单元/楼层/户号", notes = "查询 楼栋/单元/楼层/户号")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "building楼层;" +
                    "unit单元;level楼层") // ;roomNo户号
    })
    @RequestMapping(value = "/building/address",method = RequestMethod.POST)
    public ResponseBean buildingUnitNumber(@RequestBody String parameter, HttpSession session){
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.getBuildingOrUnitOrNumber(parameter, userElement.getCommunityId());
        return responseBean;
    }
    
    @ApiOperation(value = "上传楼栋信息接口", notes = "上传楼栋信息接口")
    @PostMapping(value = "/building/uploadExcel", consumes = "multipart/*", headers = "content-type=multipart/form-data")
    public ResponseBean uploadExcel(@RequestParam("file") MultipartFile file) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iBuildingOwnerInfoService.getExcelData(file);
        return responseBean;
    }
 
    @ApiOperation(value = "上传楼栋信息并入库", notes = "上传楼栋信息并入库")
    @PostMapping(value = "/building/submitExcel", consumes = "multipart/*", headers = "content-type=multipart/form-data")
    public ResponseBean submitExcel(@RequestParam("file") MultipartFile file, HttpSession session) {
        ResponseBean responseBean = new ResponseBean();
        UserElement userElement = getUserElement(session);
        responseBean = iBuildingOwnerInfoService.submitExcelData(file,userElement);
        return responseBean;
    }

    @ApiOperation(value = "获取已认证通过的用户信息",nickname ="获取已认证通过的用户信息")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="ownerName:姓名,ownerTel:手机号 ,current:每页显示多少条,current:每页显示多少条,pageSzie:当前页数")})

    @RequestMapping(value="/userPassCertification",method = RequestMethod.POST)
    public ResponseBean userPassCertification(@RequestBody String paramets,HttpSession session){
        UserElement userElement = getUserElement(session);
        ResponseBean allApprove=iBuildingOwnerInfoService.selectUserApprove(paramets,userElement.getCommunityId());
        return allApprove;
    }

    @ApiOperation(value = "修改用户当前认证状态",nickname ="修改用户当前认证状态")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataType = "String", name = "paramets", value ="ownerTel:手机号,verifyStatus:当前状态")})

    @RequestMapping(value="/updataPassCertification",method = RequestMethod.POST)
    public ResponseBean updataPassCertification(@RequestBody String paramets){
        ResponseBean userParent=iBuildingOwnerInfoService.updataApprove(paramets);
        return userParent;
    }
}
