package com.community.huiju.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.community.commom.mode.ResponseBean;
import com.community.commom.session.UserElement;
import com.community.huiju.dao.TdImagesMapper;
import com.community.huiju.dao.TpTransactionMapper;
import com.community.huiju.model.TdImages;
import com.community.huiju.model.TpBuildingOwnerInfo;
import com.community.huiju.model.TpTransaction;
import com.community.huiju.service.ITransactionService;
import com.google.common.collect.Maps;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.xml.crypto.Data;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 物业web端二手交易表 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2018-12-29
 */
@Service
public class TransactionServiceImpl extends ServiceImpl<TpTransactionMapper, TpTransaction> implements ITransactionService {
	
	@Autowired
	private TpTransactionMapper tpTransactionMapper;
	
	@Autowired
	private TdImagesMapper tdImagesMapper;
	
	@Override
	public ResponseBean getTransactionList(TpTransaction tpTransaction, Integer pageNum, Integer pageSize) {
		ResponseBean responseBean = new ResponseBean();
		Page page = new Page(pageNum,pageSize);
		// 分页查询
		List<TpTransaction> list = tpTransactionMapper.selectListByParams(page,tpTransaction);
		Map<String, Object> map = Maps.newHashMap();
		map.put("list",list);
		map.put("total",page.getTotal());
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	/**
	 * 获取交易信息详情
	 *
	 * @param id
	 * @return
	 */
	@Override
	public ResponseBean getTransactionInfo(Integer id) {
		ResponseBean responseBean = new ResponseBean();
		TpTransaction tpTransaction = tpTransactionMapper.selectById(id);
		QueryWrapper<TdImages> queryWrapper = new QueryWrapper<>();
		queryWrapper.eq("uuid", id);
		queryWrapper.eq("type", "transaction");
		List<TdImages> imagesList = tdImagesMapper.selectList(queryWrapper);
		// 转换, 只保留 imageUrl 字段
		List<String> stringList = imagesList.stream().map(e->new String(e.getImageUrl())).collect(Collectors.toList());
		Map<String,Object> map = Maps.newHashMap();
		map.put("tpTransaction",tpTransaction);
		map.put("list",stringList);
		responseBean.addSuccess(map);
		return responseBean;
	}
	
	/**
	 * 作废二手交易
	 *
	 * @param id
	 * @return
	 */
	@Override
	public ResponseBean updateTransactionById(Integer id,Integer userId) {
		ResponseBean responseBean = new ResponseBean();
		TpTransaction tpTransaction = new TpTransaction();
		tpTransaction.setId(id);
		tpTransaction.setStatus("0");
		tpTransaction.setUpdateUser(userId);
		tpTransaction.setUpdateDate(LocalDateTime.now());
		int count = tpTransactionMapper.updateById(tpTransaction);
		if (count > 0){
			responseBean.addSuccess(count);
			return responseBean;
		}
		responseBean.addError("作废出错");
		return responseBean;
	}
}
